
// https://www.shadertoy.com/view/MtGcRy

float iTime;
vec2 iResolution;

float dot2(vec2 p1, vec2 p2)
{
	return p1.x * p1.y + p2.x * p1.y;
}

const float NoiseResolution = 4.0; //  0.4
const float Lacunarity = 2.0; // 2.0 // intuitive measure of gappiness / heterogenity or variability
const float Gain = 0.6; // 0.6
const float Ball_rad = 0.45; // 0.45
const float Ball_roll_spd = 0.5; // 0.5
const float Dark_lava_spd = 0.05; // 0.05
const float Dark_island_spd = 0.5; // 0.5

// Let's get random numbers
vec2 random2D(vec2 p) {
	return fract(sin(vec2(dot2(p, vec2(127.1, 311.7)),
                          dot2(p, vec2(269.5, 183.3))))*43758.5453);
}

float random1D(vec2 p) {
    return fract(sin(dot2(p.xy,vec2(12.9898,78.233))) * 43758.5453123);
}


// Add a bit of noise
float noise2D(vec2 _pos) {	
    vec2 i = floor(_pos); 		// integer
	vec2 f = fract(_pos); 		// fraction

// define the corners of a tile
	float a = random1D(i);
	float b = random1D(i + vec2(1.0, 0.0));
	float c = random1D(i + vec2(0.0, 1.0));
	float d = random1D(i + vec2(1.0, 1.0));

	// smooth Interpolation
	vec2 u = smoothstep(0.0, 1.0, f);
    
	// lerp between the four corners
	return mix(a, b, u.x) + (c - a) * u.y * (1.0 - u.x) + (d - b) * u.x * u.y;
}

// fractal brownian motion
float fbm(vec2 _pos) {
	_pos.y += iTime * Ball_roll_spd;
	_pos.x += sin(iTime * Ball_roll_spd);
	float ts = iTime * Dark_lava_spd;
	float val = 0.0;
	float amp = 0.4;
    
	// Loop of octaves
	for (int i = 0; i < 4; ++i) // set octave number to 4
	{
		val += amp * noise2D(_pos+ts);
		_pos *= Lacunarity;
		amp *= Gain;
	}
	return val;
}

float voronoiIQ(vec2 _pos) {
	_pos.y += iTime * Ball_roll_spd;
	_pos.x += sin(iTime * Ball_roll_spd);
	vec2 p = floor(_pos);
	vec2  f = fract(_pos);
	float res = 0.0; 
	for (int j = -1; j <= 1; j++)
		for (int i = -1; i <= 1; i++)
		{
			vec2 b = vec2(i, j);
			vec2 pnt = random2D(p + b);
			pnt = 0.5 + 0.5*sin((iTime * Dark_island_spd) + 6.2831*pnt);
			vec2 r = vec2(b) - f + pnt;
			float d = dot(r, r);
			res += exp(-32.0*d); // quickly decaying exponential 
		}
   return -(1.0 / 32.0)*log(res);
}

vec4 mainImage( in vec2 inUV, in vec2 resolution )
{
    iTime = environment.time;
    iResolution = resolution;
    vec2 fragCoord = vec2(inUV.x, inUV.y);
	
	float r = 0.0;
	float g = 0.0;
	float b = 0.0;
    vec3 color = vec3(0.0, 0.0, 0.0);
    
    // Normalized pixel coordinates (from 0 to 1)
	vec2 uv = fragCoord.xy / iResolution.y * 4.0;
	vec2 pos = uv.xy - vec2(0.825, 0.5) ; // center what being drawn

    color.rg = vec2(voronoiIQ(pos.xy));
    color.r += 0.25+fbm(pos.xy);

    // Output to screen
    return vec4(color,1.0);
}
